@@ -5,6 +5,8 @@ import (
 	"fmt"
 	"reflect"
 
+	operatorcontroller "github.com/openshift/cluster-ingress-operator/pkg/operator/controller"
+
 	operatorv1 "github.com/openshift/api/operator/v1"
 	"github.com/openshift/cluster-ingress-operator/pkg/manifests"
 	"github.com/openshift/cluster-ingress-operator/pkg/operator/controller"
@@ -30,20 +32,21 @@ func (r *reconciler) ensureServiceMonitor(ic *operatorv1.IngressController, svc
 
 	switch {
 	case !haveSM:
-		if created, err := r.createServiceMonitor(desired); err != nil {
+		if err := r.client.Create(context.TODO(), desired); err != nil {
 			return false, nil, fmt.Errorf("failed to create servicemonitor %s/%s: %v", desired.GetNamespace(), desired.GetName(), err)
-		} else if created {
-			log.Info("created servicemonitor", "namespace", desired.GetNamespace(), "name", desired.GetName())
 		}
+		log.Info("created servicemonitor", "namespace", desired.GetNamespace(), "name", desired.GetName())
+		return r.currentServiceMonitor(ic)
 	case haveSM:
 		if updated, err := r.updateServiceMonitor(current, desired); err != nil {
 			return true, current, fmt.Errorf("failed to update servicemonitor %s/%s: %v", desired.GetNamespace(), desired.GetName(), err)
 		} else if updated {
 			log.Info("updated servicemonitor", "namespace", desired.GetNamespace(), "name", desired.GetName())
+			return r.currentServiceMonitor(ic)
 		}
 	}
 
-	return r.currentServiceMonitor(ic)
+	return true, current, nil
 }
 
 // desiredServiceMonitor returns the desired servicemonitor for the given
@@ -59,7 +62,7 @@ func desiredServiceMonitor(ic *operatorv1.IngressController, svc *corev1.Service
 			"spec": map[string]interface{}{
 				"namespaceSelector": map[string]interface{}{
 					"matchNames": []interface{}{
-						"openshift-ingress",
+						operatorcontroller.DefaultOperandNamespace,
 					},
 				},
 				"selector": map[string]interface{}{
@@ -120,15 +123,6 @@ func (r *reconciler) currentServiceMonitor(ic *operatorv1.IngressController) (bo
 	return true, sm, nil
 }
 
-// createServiceMonitor creates a servicemonitor.  Returns a Boolean indicating
-// whether the servicemonitor was created, and an error value.
-func (r *reconciler) createServiceMonitor(sm *unstructured.Unstructured) (bool, error) {
-	if err := r.client.Create(context.TODO(), sm); err != nil {
-		return false, err
-	}
-	return true, nil
-}
-
 // updateServiceMonitor updates a servicemonitor.  Returns a Boolean indicating
 // whether the servicemonitor was updated, and an error value.
 func (r *reconciler) updateServiceMonitor(current, desired *unstructured.Unstructured) (bool, error) {
