@@ -28,6 +28,7 @@ import (
 	"github.com/openshift/cluster-ingress-operator/pkg/manifests"
 	operatorclient "github.com/openshift/cluster-ingress-operator/pkg/operator/client"
 	"github.com/openshift/cluster-ingress-operator/pkg/operator/controller"
+	operatorcontroller "github.com/openshift/cluster-ingress-operator/pkg/operator/controller"
 	ingresscontroller "github.com/openshift/cluster-ingress-operator/pkg/operator/controller/ingress"
 
 	"github.com/aws/aws-sdk-go/aws/endpoints"
@@ -56,7 +57,7 @@ import (
 )
 
 var (
-	defaultAvailableConditions = []operatorv1.OperatorCondition{
+	nondefaultAvailableConditions = []operatorv1.OperatorCondition{
 		{Type: operatorv1.IngressControllerAvailableConditionType, Status: operatorv1.ConditionTrue},
 		{Type: operatorv1.LoadBalancerManagedIngressConditionType, Status: operatorv1.ConditionTrue},
 		{Type: operatorv1.LoadBalancerReadyIngressConditionType, Status: operatorv1.ConditionTrue},
@@ -64,12 +65,14 @@ var (
 		{Type: operatorv1.DNSReadyIngressConditionType, Status: operatorv1.ConditionTrue},
 		{Type: ingresscontroller.IngressControllerAdmittedConditionType, Status: operatorv1.ConditionTrue},
 	}
+	// The ingress canary check status condition only applies to the default ingress controller.
+	defaultAvailableConditions = append(nondefaultAvailableConditions, operatorv1.OperatorCondition{Type: ingresscontroller.IngressControllerCanaryCheckSuccessConditionType, Status: operatorv1.ConditionTrue})
 )
 
 var kclient client.Client
 var dnsConfig configv1.DNS
 var infraConfig configv1.Infrastructure
-var operatorNamespace = manifests.DefaultOperatorNamespace
+var operatorNamespace = operatorcontroller.DefaultOperatorNamespace
 var defaultName = types.NamespacedName{Namespace: operatorNamespace, Name: manifests.DefaultIngressControllerName}
 
 func TestMain(m *testing.M) {
@@ -106,6 +109,47 @@ func TestOperatorSteadyConditions(t *testing.T) {
 	}
 }
 
+func TestClusterOperatorStatusRelatedObjects(t *testing.T) {
+	expected := []configv1.ObjectReference{
+		{
+			Resource: "namespaces",
+			Name:     operatorNamespace,
+		},
+		{
+			Group:     operatorv1.GroupName,
+			Resource:  "IngressController",
+			Namespace: operatorNamespace,
+		},
+		{
+			Group:     iov1.GroupVersion.Group,
+			Resource:  "DNSRecord",
+			Namespace: operatorNamespace,
+		},
+		{
+			Resource: "namespaces",
+			Name:     "openshift-ingress",
+		},
+		{
+			Resource: "namespaces",
+			Name:     "openshift-ingress-canary",
+		},
+	}
+
+	coName := controller.IngressClusterOperatorName()
+	err := wait.PollImmediate(1*time.Second, 5*time.Minute, func() (bool, error) {
+		co := &configv1.ClusterOperator{}
+		if err := kclient.Get(context.TODO(), coName, co); err != nil {
+			t.Logf("failed to get ingress cluster operator %s: %v", coName, err)
+			return false, nil
+		}
+
+		return reflect.DeepEqual(expected, co.Status.RelatedObjects), nil
+	})
+	if err != nil {
+		t.Errorf("did not get expected status related objects: %v", err)
+	}
+}
+
 func TestDefaultIngressControllerSteadyConditions(t *testing.T) {
 	if err := waitForIngressControllerCondition(t, kclient, 10*time.Second, defaultName, defaultAvailableConditions...); err != nil {
 		t.Errorf("did not get expected conditions: %v", err)
@@ -120,14 +164,14 @@ func TestUserDefinedIngressController(t *testing.T) {
 	}
 	defer assertIngressControllerDeleted(t, kclient, ing)
 
-	if err := waitForIngressControllerCondition(t, kclient, 5*time.Minute, name, defaultAvailableConditions...); err != nil {
+	if err := waitForIngressControllerCondition(t, kclient, 5*time.Minute, name, nondefaultAvailableConditions...); err != nil {
 		t.Errorf("failed to observe expected conditions: %v", err)
 	}
 }
 
 func TestUniqueDomainRejection(t *testing.T) {
 	def := &operatorv1.IngressController{}
-	if err := waitForIngressControllerCondition(t, kclient, 5*time.Minute, defaultName, defaultAvailableConditions...); err != nil {
+	if err := waitForIngressControllerCondition(t, kclient, 5*time.Minute, defaultName, nondefaultAvailableConditions...); err != nil {
 		t.Fatalf("failed to observe expected conditions: %v", err)
 	}
 
@@ -162,7 +206,7 @@ func TestClusterProxyProtocol(t *testing.T) {
 	if err := kclient.Get(context.TODO(), defaultName, ic); err != nil {
 		t.Fatalf("failed to get default ingresscontroller: %v", err)
 	}
-	if err := waitForIngressControllerCondition(t, kclient, 5*time.Minute, defaultName, defaultAvailableConditions...); err != nil {
+	if err := waitForIngressControllerCondition(t, kclient, 5*time.Minute, defaultName, nondefaultAvailableConditions...); err != nil {
 		t.Fatalf("failed to observe expected conditions: %v", err)
 	}
 
@@ -194,7 +238,7 @@ func TestUpdateDefaultIngressController(t *testing.T) {
 	if err := kclient.Get(context.TODO(), defaultName, ic); err != nil {
 		t.Fatalf("failed to get default ingresscontroller: %v", err)
 	}
-	if err := waitForIngressControllerCondition(t, kclient, 5*time.Minute, defaultName, defaultAvailableConditions...); err != nil {
+	if err := waitForIngressControllerCondition(t, kclient, 5*time.Minute, defaultName, nondefaultAvailableConditions...); err != nil {
 		t.Fatalf("failed to observe expected conditions: %v", err)
 	}
 
@@ -307,7 +351,7 @@ func TestIngressControllerScale(t *testing.T) {
 	if err := kclient.Get(context.TODO(), defaultName, ic); err != nil {
 		t.Fatalf("failed to get default ingresscontroller: %v", err)
 	}
-	if err := waitForIngressControllerCondition(t, kclient, 5*time.Minute, defaultName, defaultAvailableConditions...); err != nil {
+	if err := waitForIngressControllerCondition(t, kclient, 5*time.Minute, defaultName, nondefaultAvailableConditions...); err != nil {
 		t.Fatalf("failed to observe expected conditions: %v", err)
 	}
 
@@ -359,7 +403,7 @@ func TestIngressControllerScale(t *testing.T) {
 	}
 
 	// Ensure the ingresscontroller remains available
-	if err := waitForIngressControllerCondition(t, kclient, 2*time.Minute, defaultName, defaultAvailableConditions...); err != nil {
+	if err := waitForIngressControllerCondition(t, kclient, 2*time.Minute, defaultName, nondefaultAvailableConditions...); err != nil {
 		t.Fatalf("failed to observe expected conditions: %v", err)
 	}
 
@@ -384,7 +428,7 @@ func TestIngressControllerScale(t *testing.T) {
 
 	// Ensure the ingresscontroller remains available
 	// TODO: assert that the conditions hold steady for some amount of time?
-	if err := waitForIngressControllerCondition(t, kclient, 5*time.Minute, defaultName, defaultAvailableConditions...); err != nil {
+	if err := waitForIngressControllerCondition(t, kclient, 5*time.Minute, defaultName, nondefaultAvailableConditions...); err != nil {
 		t.Fatalf("failed to observe expected conditions: %v", err)
 	}
 
@@ -440,7 +484,7 @@ func TestDefaultIngressCertificate(t *testing.T) {
 		return
 	}
 
-	if err := waitForIngressControllerCondition(t, kclient, 5*time.Minute, defaultName, defaultAvailableConditions...); err != nil {
+	if err := waitForIngressControllerCondition(t, kclient, 5*time.Minute, defaultName, nondefaultAvailableConditions...); err != nil {
 		t.Fatalf("failed to observe expected conditions: %v", err)
 	}
 
@@ -506,7 +550,7 @@ func TestPodDisruptionBudgetExists(t *testing.T) {
 		t.Fatalf("failed to get default ingresscontroller: %v", err)
 	}
 
-	if err := waitForIngressControllerCondition(t, kclient, 5*time.Minute, defaultName, defaultAvailableConditions...); err != nil {
+	if err := waitForIngressControllerCondition(t, kclient, 5*time.Minute, defaultName, nondefaultAvailableConditions...); err != nil {
 		t.Fatalf("failed to observe expected conditions: %v", err)
 	}
 
@@ -569,7 +613,7 @@ func TestInternalLoadBalancer(t *testing.T) {
 	defer assertIngressControllerDeleted(t, kclient, ic)
 
 	// Wait for the load balancer and DNS to be ready.
-	if err := waitForIngressControllerCondition(t, kclient, 5*time.Minute, name, defaultAvailableConditions...); err != nil {
+	if err := waitForIngressControllerCondition(t, kclient, 5*time.Minute, name, nondefaultAvailableConditions...); err != nil {
 		t.Fatalf("failed to observe expected conditions: %v", err)
 	}
 
@@ -1153,7 +1197,7 @@ func TestIngressControllerCustomEndpoints(t *testing.T) {
 		t.Skipf("skipping TestIngressControllerCustomEndpoints test due to platform type: %s", platform.Type)
 	}
 	// The default ingresscontroller should surface the expected status conditions.
-	if err := waitForIngressControllerCondition(t, kclient, 30*time.Second, defaultName, defaultAvailableConditions...); err != nil {
+	if err := waitForIngressControllerCondition(t, kclient, 30*time.Second, defaultName, nondefaultAvailableConditions...); err != nil {
 		t.Fatalf("did not get expected ingress controller conditions: %v", err)
 	}
 	// Ensure an ingresscontroller can be created with custom endpoints.
@@ -1164,7 +1208,7 @@ func TestIngressControllerCustomEndpoints(t *testing.T) {
 	}
 	defer assertIngressControllerDeleted(t, kclient, ic)
 	// Ensure the ingress controller is reporting expected status conditions.
-	if err := waitForIngressControllerCondition(t, kclient, 5*time.Minute, name, defaultAvailableConditions...); err != nil {
+	if err := waitForIngressControllerCondition(t, kclient, 5*time.Minute, name, nondefaultAvailableConditions...); err != nil {
 		t.Errorf("failed to observe expected conditions: %v", err)
 	}
 }
@@ -1313,9 +1357,13 @@ func TestHTTPCookieCapture(t *testing.T) {
 				Type:      operatorv1.ContainerLoggingDestinationType,
 				Container: &operatorv1.ContainerLoggingDestinationParameters{},
 			},
-			HTTPCaptureCookies: []operatorv1.IngressControllerCaptureHTTPCookie{
-				{MatchType: "Exact", Name: "foo", MaxLength: 9},
-			},
+			HTTPCaptureCookies: []operatorv1.IngressControllerCaptureHTTPCookie{{
+				IngressControllerCaptureHTTPCookieUnion: operatorv1.IngressControllerCaptureHTTPCookieUnion{
+					MatchType: "Exact",
+					Name:      "foo",
+				},
+				MaxLength: 9,
+			}},
 		},
 	}
 	if err := kclient.Create(context.TODO(), ic); err != nil {
@@ -1460,7 +1508,7 @@ func TestNetworkLoadBalancer(t *testing.T) {
 	defer assertIngressControllerDeleted(t, kclient, ic)
 
 	// Wait for the load balancer and DNS to be ready.
-	if err := waitForIngressControllerCondition(t, kclient, 5*time.Minute, name, defaultAvailableConditions...); err != nil {
+	if err := waitForIngressControllerCondition(t, kclient, 5*time.Minute, name, nondefaultAvailableConditions...); err != nil {
 		t.Fatalf("failed to observe expected conditions: %v", err)
 	}
 
